#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File  : mqBios_v.py
# Author: My Duan
# Date  : 2022-03-03


import numpy as np
import pandas as pd
from scipy.stats import pearsonr
import joblib
import warnings
import tqdm
import os
from collections import defaultdict
from sklearn.linear_model import LinearRegression
warnings.filterwarnings(action='ignore')

egFileCoef = "ModelCoef.csv"
egFileMatrix_mqTrans = "Matrix_mqTrans.csv"
egFileMatrix_mRNA = "Matrix_mRNA.csv"
egFilePCC = "mqTrans_Features_PCC.csv"
egFileModel = "mqTransModels.pkl"


def model(trainGene_file, trainTF_file, testGene_file, testTF_file, geneList_file, egFolderO):

	# coef_folder = egFolderO + "\\" + "coef"
	# pkl_folder = egFolderO + "\\" + "model"

	# if not os.path.exists(coef_folder):
	# 	os.makedirs(coef_folder)
	# else: pass
	#
	# if not os.path.exists(pkl_folder):
	# 	os.makedirs(pkl_folder)
	# else: pass

	train_gene_dataset = pd.read_csv(trainGene_file,index_col=0)
	train_TF_dataset = pd.read_csv(trainTF_file,index_col=0)
	test_gene_dataset = pd.read_csv(testGene_file,index_col=0)
	test_TF_dataset = pd.read_csv(testTF_file,index_col=0)

	genes = pd.read_csv(geneList_file,index_col=0)

	assert list(test_gene_dataset.columns) ==  list(test_TF_dataset.columns)
	samples = list(test_TF_dataset.columns)

	re_teseGene = test_gene_dataset.copy()

	indexes = list(train_TF_dataset.index)
	# print(train_TF_dataset.index)
	train_gene_dataset = np.array(train_gene_dataset)
	train_TF_dataset = np.array(train_TF_dataset)
	test_gene_dataset = np.array(test_gene_dataset)
	test_TF_dataset = np.array(test_TF_dataset)

	name_pcc=[]
	number_pcc=[]
	count=0
	pcc_matrix=[]
	GENE_name=[]
	abs_matrix=[]
	# ========Linear Regression========

	print('\n Model training ...\n')

	coef_vectors = []
	intercept_vectors = []

	model_pkls = defaultdict(list)

	demo_num = 100
	for i in tqdm.tqdm(range(genes.shape[0])[:demo_num], ascii=True):

		X_train = train_TF_dataset[:,:].T
		y_train = train_gene_dataset[i]
		X_test = test_TF_dataset[::].T
		y_test = test_gene_dataset[i]

		weights = abs(LinearRegression().fit(X_train, y_train).coef_)
		selected_index = np.argwhere(weights >= max(weights) * 0.9).reshape(1, -1)[0]
		coef = np.zeros(len(weights))
		clf = LinearRegression().fit(X_train[:, selected_index], y_train)
		select_coef = clf.coef_
		intercept = clf.intercept_
		# print(selected_index)
		# print("Intercept", intercept, ",Cofficient：", select_coef)
		for j  in range(len(selected_index)):
			coef[selected_index[j]] = select_coef[j]
		pre_valid_y = clf.predict(X_test[:, selected_index])

		# PCC threshold = 0.5
		pcc_vector = pearsonr(y_test, pre_valid_y)
		mqTrans = abs(y_test - pre_valid_y)

		if pcc_vector[0] > 0.5:
			number_pcc.append(i)
			name_pcc.append(genes.index[i])

			coef_vectors.append(coef)
			intercept_vectors.append(intercept)

			pcc_matrix.append(pcc_vector[0])
			abs_matrix.append(mqTrans)
			GENE_name.append(genes.index[i])

			model_pkls[genes.index[i]] = clf

			count += 1

	indexes.append('Intercept')
	coef_matrix = pd.DataFrame(np.vstack((np.array(coef_vectors).T, np.array(intercept_vectors).reshape(1,-1))),
						   columns = name_pcc, index = indexes)

	#coef_path = egFolderO + "\\" + "coef.csv"
	coef_path = egFolderO + "\\" + egFileCoef
	coef_matrix.to_csv(coef_path)

	#pkl_path = egFolderO + "\\" + "model.pkl"
	pkl_path = egFolderO + "\\" + egFileModel
	joblib.dump(model_pkls, pkl_path)

	print('\n The number of models saved after calculating PCC: ', count)

	pcc_df = pd.DataFrame(np.array(pcc_matrix).T, index = name_pcc, columns=['PearsonValue'])
	abs_df = pd.DataFrame(np.array(abs_matrix).T, samples, columns=name_pcc)


	#pcc_df.to_csv(egFolderO + "\\" + "matrix_pcc.csv")
	pcc_df.to_csv(egFolderO + "\\" + egFilePCC)
	#abs_df.to_csv(egFolderO + "\\" + "matrix_abs.csv")
	abs_df.to_csv(egFolderO + "\\" + egFileMatrix_mqTrans)

	new_dataset = re_teseGene.loc[GENE_name,:]
	#new_dataset.to_csv(egFolderO + "\\" + "pcc_after_test_gene_line_GSE33356newset.csv")
	new_dataset.to_csv(egFolderO + "\\" + egFileMatrix_mRNA)

	return
